#!/bin/sh
#
# Script to start DataConversion tool
#
#  Available commandline arguments:
#  regionpath=path or name of region folder. Required parameter
#  configfile=configuration file. Optional parameter. If not used then parameter workflowid is required.
#  systemtime=system time (yyyy-MM-dd HH:mm:ssZ) to use when running. Optional parameter. Defaults to current system time.
#  importpath=path to import files. Used in combination with parameter workflowid.
#  exportpath=path to export files. Used in combination with parameter workflowid.
#  binpath=path to binaries. Optional parameter. If omitted the DCM must be run from within the bin folder.
#
#.
# Example usages:
# DataConversion regionpath=%1 configfile=%2 systemtime=%3
# DataConversion regionpath=%1 workflowid=%2 importpath=%3 exportpath=%4 systemtime=%5#
# File history
# Version           Date                           Author
# $Revision: 39313 $  $Date: 2013-02-13 17:05:22 +0100 (Wed, 13 Feb 2013) $   $Author: rooij_e $

# Path to the JRE
# export JAVA_HOME=/usr

# Remote debugging: uncomment the following line:
#JVM_OPTS="$JVM_OPTS -Xdebug -Xrunjdwp:transport=dt_socket,address=8800,suspend=y,server=y"

BATCHDIR=$(pwd)
JAVA_HOME=$BATCHDIR/jre1.7.0_80
BINPATH=$BATCHDIR/bin
# Check if the Bin dir is set
if [ wl"$BATCHDIR" = "wl" ]; then
  echo ERROR: $BINPATH not found, please make sure that
  echo the variable BINPATH in this script points to the directory containing the DCM binaries.
  exit 1
fi

# Application must be started from within the /bin/ directory
echo "Working directory is: BINPATH"
cd $BINPATH || exit 1

# Assemble the list of jar files in the bin directory
classes=""
for f in $BINPATH/*.jar ;do
    if [ "$classes" = "" ]; then
        classes="$f"
    else
        classes="${classes}:$f"
    fi
done

echo "Using Java runtime: ${JAVA_HOME:-/opt/fews/java}/bin/java - should be at least version 1.7.0_11"
echo "Starting data conversion ..."

${JAVA_HOME:-/opt/fews/java}/bin/java -Xmx512M -XX:-PrintWarnings $JVM_OPTS -Djava.library.path=$BINPATH \
-classpath "$classes" \
nl.wldelft.dataconversion.DataConversionModule "$@"

exit
