#!/usr/bin/python3

# -*- coding: utf-8 -*-
"""
Created on Tue Jun 23 16:11:28 2015

@author: hendrik_gt
"""

"""
Load data from FTP, transform it, save to csv and let FEWS load the data.
"""

#  Copyright notice
#   --------------------------------------------------------------------
#   Copyright (C) 2015 Deltares for IWRAP
#       Gerrit Hendriksen
#
#       gerrit.hendriksen@deltares.nl
#
#   This library is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This library is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this library.  If not, see <http://www.gnu.org/licenses/>.
#   --------------------------------------------------------------------
#
# This tool is part of <a href="http://www.OpenEarth.eu">OpenEarthTools</a>.
# OpenEarthTools is an online collaboration to share and manage data and
# programming tools in an open source, version controlled environment.
# Sign up to recieve regular updates of this function, and to contribute
# your own tools.

# $Id: odv2orm_initialize.py 11010 2014-07-30 10:03:46Z boer_g $
# $Date: 2014-07-30 12:03:46 +0200 (wo, 30 jul 2014) $
# $Author: boer_g $
# $Revision: 11010 $
# $HeadURL: https://svn.oss.deltares.nl/repos/openearthtools/trunk/python/OpenEarthTools/openearthtools/io/pyodv/odv2orm_initialize.py $
# $Keywords: $

import os
from ftplib import FTP
import csv

cf = os.path.join(os.path.dirname(os.path.abspath(__file__)),"ftpcredentials.txt")

lfout = os.path.join(os.path.dirname(os.path.abspath(__file__)),"diag.xml")

lf = open(lfout,'wb')
lf.write("""<?xml version="1.0" encoding="UTF-8" standalone="yes"?>"""+'\r\n'+'\r\n')
lf.write("""<Diag xmlns="http://www.wldelft.nl/fews/PI" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.wldelft.nl/fews/PI HTTP://fews.wldelft.nl/schemas/version1.0/pi-schemas/pi_diag.xsd" version="1.2">"""+'\r\n')
lf.write("""    <line level="3" description="Convert script has been activated."></line>"""+'\r\n')
lf.write("""</Diag>"""+'\r\n')
lf.close

def credentials():
    cr = get_credentials(cf)
    return cr

def get_credentials(credentialfile):
    fdbp = open(credentialfile,'rb')
    credentials = {}
    for i in fdbp:
        item = i.split('=')
        if str.strip(item[0]) == 'url':
            credentials['url'] = str.strip(item[1])
        if str.strip(item[0]) == 'uname':
            credentials['user'] = str.strip(item[1])
        if str.strip(item[0]) == 'password':
            credentials['password'] = str.strip(item[1])
        if str.strip(item[0]) == 'writedir':
            credentials['writedir'] = str.strip(item[1])
        if str.strip(item[0]) == 'directory':
            if str.strip(item[1]) != '':
                credentials['directory'] = str.strip(item[1])
    return credentials

def downloadfromftp(crd):
    ftp = FTP(crd['url'],crd['user'],crd['password'])
    if crd['directory'] != '':
        ftp.cwd(crd['directory'])
    filenames = ftp.nlst()
    
    for filename in filenames:
        local_filename = os.path.join(crd['writedir'], filename)
        try:
            file = open(local_filename, 'wb')
            ftp.retrbinary('RETR '+ filename, file.write)
            file.close()
            # immediately transform the file to FEWS_CSV and transform Puls to mm rain (*0.2)
            statsize = os.stat(local_filename).st_size
            if statsize > 0:
                print ("File: " + local_filename + "\nsize: " + str(statsize) + " bytes")
                transform2csv(crd,local_filename)
        except Exception as err:
            print(str(err) + ".")
        finally:
            # remove filename.txt that has now been copied and converted to filename.csv
            os.unlink(local_filename)    
    ftp.quit()

def transform2csv(crd,local_filename):
    fout = local_filename.replace('.txt','.csv')
    csvfile = open(local_filename) # parse input file headers
    sample = csvfile.read(1024)
    csvfile.seek(0)
    dialect = csv.Sniffer().sniff(sample)
    header = ''
    linereader = csv.reader(csvfile, dialect)
    if csv.Sniffer().has_header(sample):
        header = next(linereader)
    f = open(fout,'wb') # create output file
    f.write(','.join(['Location Names',local_filename.split('-')[1]])+'\r\n')
    f.write(','.join(['Location Ids',local_filename.split('-')[1]])+'\r\n')
    f.write(','.join(['Time','P.obs'])+'\r\n')
    # NOTE!!
    # If the step that follows this script (configured FEWS workflow Import_CSV) does not import any data, the reason is most likely that the locationid of the ElliTrack instrument (e.g. 14022401 or 14022402 or 14022403) is not in the locations table of the FEWS database. FEWS ignores importing data for locations that it doesn't know.
    for row in linereader: # read input file, write output file
        time = row[0]
        value = row[1]
        if value != '':
            value = float(value)
            value = value * 0.2
            value = str(value)
        else:
            value = 'NaN' # FEWS Nodata
        f.write(time + "," + value + "\r\n")
    f.close()
    
if __name__ == '__main__':   
    ## make connection to FTP and create a list of files
    crd = credentials()
    downloadfromftp(crd)
    
    # call FEWS import module
    fewsdir = crd['writedir']
    
    
    
    
def tests():    
    import glob
    for local_filename in glob.glob(r'D:\temp\ellitrack\*.txt'):
        transform2csv(crd,local_filename)





