-- ================================================================
--  Delft FEWS
--  ================================================================
-- 
--  Project Info:  http://www.wldelft.nl/soft/fews/index.html
--  Project Lead:  Karel Heynert (karel.heynert@wldelft.nl)
-- 
--  (C) Copyright 2003, by WL | Delft Hydraulics
--                         P.O. Box 177
--                         2600 MH  Delft
--                         The Netherlands
--                         http://www.wldelft.nl
-- 
--  DELFT-FEWS is a sophisticated collection of modules designed
--  for building a FEWS customised to the specific requirements
--  of individual agencies. An open modelling approach allows users
--  to add their own modules in an efficient way.
-- 
--  ----------------------------------------------------------------
--  local_master_datastore_isosql99.sql
--  ----------------------------------------------------------------
--  (C) Copyright 2003-2007, by WL | Delft Hydraulics
-- 
-- File history
-- Version           Date                           Author
-- $Revision: 43516 $  $Date: 2013-11-28 14:54:58 +0100 (Thu, 28 Nov 2013) $   $Author: breugel $
--

-- Only use ISO SQL99 statements and keywords
-- Booleans are not allowed because the are not supported by Oracle and because Tessella makes
-- no difference between ints and booleans at a lower level so it is not easy to support this.

-- todos
-- ModuleInstanceRuns
-- add Expiry time DONE

-- CurrentModuleInstanceRuns
-- add Expiry time (expiry time is always the same as the referenced module instance run)
-- comment out remove FK (access does not supports a combined foreignkey with nullable fields)

--TaskRunCompletions
-- rename modification time to creation time
-- add expiryTime DONE

-- SystemActivities
-- add creationTime
-- add expiryTime DONE

-- TimeSeriesTable
-- remove taskrun
-- make ensembleId nullable

-- FewsSessions
-- add expiry time DONE

-  CurrentModuleInstanceRuns
-- add userId column not null the

--TODO Implement ; as closing sign for sql statement iso empty line



-- only used by MC

#skipCreateExistingTables

CREATE TABLE Sequences(
    entryId INTEGER NOT NULL,
    mcId VARCHAR(64) NOT NULL,
    nextTaskId INTEGER DEFAULT 0 NOT NULL,
    nextTaskRunId INTEGER DEFAULT 0 NOT NULL,
    nextSystemStatusDigestId INTEGER DEFAULT 0 NOT NULL,
    nextRevisionId INTEGER DEFAULT 0 NOT NULL,
    CONSTRAINT PK_Sequences
        PRIMARY KEY(entryId))

#ifColumnMissing Sequences nextRevisionId
    ALTER TABLE Sequences ADD nextRevisionId INTEGER DEFAULT 0 NOT NULL
#endif

-- a row is always immutable
#ifTableMissing DatabaseCreateTime
CREATE TABLE DatabaseCreateTime(DatabaseCreateTime TIMESTAMP)
#endif

-- implement it later instead of DatabaseCreateTime
-- #ifTableMissing DatabaseVersion
-- CREATE TABLE DatabaseVersion(Version INTEGER DEFAULT 0 NOT NULL)
--
--INSERT INTO DatabaseVersion VALUES(0)
-- #endif


-- a row is always immutable
#ifTableMissing  ModuleInstanceDatasets
CREATE TABLE ModuleInstanceDatasets(
    moduleInstanceId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ModuleInstanceDatasets
        PRIMARY KEY(moduleInstanceId, version))

CREATE INDEX ModuleInstanceDatasets_lat ON ModuleInstanceDatasets (localAvailableTime)
#endif

#ifColumnMissing ModuleInstanceDatasets synchLevel
    ALTER TABLE ModuleInstanceDatasets ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

-- a row is immutable
#ifTableMissing DefaultModuleInstanceDatasets
CREATE TABLE DefaultModuleInstanceDatasets(
    moduleInstanceId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefModuleInstanceDatasets
        PRIMARY KEY(moduleInstanceId))
--    CONSTRAINT FK_DefModuleInstanceDatasets
--        FOREIGN KEY(moduleInstanceId, defaultVersion)
--        REFERENCES ModuleInstanceDatasets(moduleInstanceId, version))

CREATE INDEX DefModuleInstanceDatasets_lat ON DefaultModuleInstanceDatasets (localAvailableTime)
#endif


#ifColumnExists MapLayers resourceId
    DROP TABLE MapLayers

    DROP TABLE DefaultMapLayers
#endif

-- a row is immutable
#ifTableMissing  MapLayers
CREATE TABLE MapLayers(
    mapLayerId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_MapLayers
        PRIMARY KEY(mapLayerId, version))

CREATE INDEX MapLayers_lat ON MapLayers (localAvailableTime)
#endif

#ifColumnMissing MapLayers synchLevel
    ALTER TABLE MapLayers ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

-- a row is mutable
#ifTableMissing DefaultMapLayers
CREATE TABLE DefaultMapLayers(
    mapLayerId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefMapLayers
        PRIMARY KEY(mapLayerId))
--    CONSTRAINT FK_DefMapLayers
--        FOREIGN KEY(mapLayerId, defaultVersion)
--        REFERENCES MapLayers(mapLayerId, version))

CREATE INDEX DefMapLayers_lat ON DefaultMapLayers (localAvailableTime)
#endif

#ifColumnExists Icons resourceId
    DROP TABLE Icons
    
    DROP TABLE DefaultIcons
#endif

-- a row is immutable
#ifTableMissing  Icons
CREATE TABLE Icons(
    iconId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_Icons
        PRIMARY KEY(iconId, version))

CREATE INDEX Icons_lat ON Icons (localAvailableTime)
#endif

#ifColumnMissing Icons synchLevel
    ALTER TABLE Icons ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

-- a row is mutable
#ifTableMissing DefaultIcons
CREATE TABLE DefaultIcons(
    iconId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefIcons
        PRIMARY KEY(iconId))
--    CONSTRAINT FK_DefIcons
--        FOREIGN KEY(iconId, defaultVersion)
--        REFERENCES Icons(iconId, version))

CREATE INDEX DefIcons_lat ON DefaultIcons (localAvailableTime)
#endif

#iftableexists Resources
    DROP TABLE Resources

    DROP TABLE DefaultResources
#endif

-- a row is immutable
#ifTableMissing  RootConfigFiles
CREATE TABLE RootConfigFiles(
    rootConfigFileId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_RootConfigFiles
        PRIMARY KEY(rootConfigFileId, version))

CREATE INDEX RootConfigFiles_lat ON RootConfigFiles (localAvailableTime)
#endif

#ifColumnMissing RootConfigFiles synchLevel
    ALTER TABLE RootConfigFiles ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

-- a row is mutable
#ifTableMissing DefaultRootConfigFiles
CREATE TABLE DefaultRootConfigFiles(
    rootConfigFileId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefRootConfigFiles
        PRIMARY KEY(rootConfigFileId))
--    CONSTRAINT FK_DefRootConfigFiles
--        FOREIGN KEY(rootConfigFileId, defaultVersion)
--        REFERENCES RootConfigFiles(rootConfigFileId, version))

CREATE INDEX DefRootConfigFiles_lat ON DefaultRootConfigFiles (localAvailableTime)
#endif

#ifTableMissing PiServiceConfigurations
CREATE TABLE PiServiceConfigurations(
    clientId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_PiServiceConfigurations
        PRIMARY KEY(clientId, version))

CREATE INDEX PiServiceConfigurations_lat ON PiServiceConfigurations (localAvailableTime)
#endif

CREATE TABLE DefaultPiServiceConfigurations(
    clientId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefPiServiceConfigurations
        PRIMARY KEY(clientId))
--    CONSTRAINT FK_DefPiServiceConfigurations
--        FOREIGN KEY(clientId, defaultVersion)
--        REFERENCES PiServiceConfigurations(clientId, version))

#ifTableMissing PiClientConfigurations
CREATE TABLE PiClientConfigurations(
    clientId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_PiClientConfigurations
        PRIMARY KEY(clientId, version))

CREATE INDEX PiClientConfigurations_lat ON PiClientConfigurations (localAvailableTime)
#endif

CREATE TABLE DefaultPiClientConfigurations(
    clientId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefPiClientConfigurations
        PRIMARY KEY(clientId))
--    CONSTRAINT FK_DefPiClientConfigurations
--        FOREIGN KEY(clientId, defaultVersion)
--        REFERENCES PiClientConfigurations(clientId, version))

-- a row is always immutable
#ifTableMissing ModuleInstanceConfigs
CREATE TABLE ModuleInstanceConfigs(
    moduleInstanceId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ModuleInstanceConfigs
        PRIMARY KEY(moduleInstanceId, version))

CREATE INDEX ModuleInstanceConfigs_lat ON ModuleInstanceConfigs (localAvailableTime)
#endif

#ifColumnMissing ModuleInstanceConfigs synchLevel
    ALTER TABLE ModuleInstanceConfigs ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

#ifTableMissing DefaultModuleInstanceConfigs
CREATE TABLE DefaultModuleInstanceConfigs(
    moduleInstanceId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefModuleInstanceConfigs
        PRIMARY KEY(moduleInstanceId))
--    CONSTRAINT FK_DefModuleInstanceConfigs
--            FOREIGN KEY(moduleInstanceId, defaultVersion)
--            REFERENCES ModuleInstanceConfigs(moduleInstanceId, version))

CREATE INDEX DefModuleInstanceConfigs_lat ON DefaultModuleInstanceConfigs (localAvailableTime)
#endif

-- a row is always immutable
#ifTableMissing SystemConfigurations
CREATE TABLE SystemConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SystemConfigurations
        PRIMARY KEY(configTypeId, version))

CREATE INDEX SystemConfigurations_lat ON SystemConfigurations (localAvailableTime)
#endif

#ifColumnMissing SystemConfigurations synchLevel
    ALTER TABLE SystemConfigurations ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif


CREATE TABLE DefaultSystemConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefSystemConfigurations
        PRIMARY KEY(configTypeId))
--    CONSTRAINT FK_DefSystemConfigurations
--        FOREIGN KEY(configTypeId, defaultVersion)
--        REFERENCES SystemConfigurations(configTypeId, version))


-- a row is always immutable
#ifTableMissing RegionConfigurations
CREATE TABLE RegionConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_RegionConfigurations
        PRIMARY KEY(configTypeId, version))

CREATE INDEX RegionConfigurations_lat ON RegionConfigurations (localAvailableTime)
#endif

#ifColumnMissing RegionConfigurations synchLevel
    ALTER TABLE RegionConfigurations ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif


CREATE TABLE DefaultRegionConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefRegionConfigurations
        PRIMARY KEY(configTypeId))
--    CONSTRAINT FK_DefRegionConfigurations
--        FOREIGN KEY(configTypeId, defaultVersion)
--        REFERENCES RegionConfigurations(configTypeId, version))

-- a row is always immutable
CREATE TABLE ModuleParameters(
    moduleParameterId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ModuleParameters
        PRIMARY KEY(moduleParameterId, version))

#ifColumnMissing ModuleParameters synchLevel
    ALTER TABLE ModuleParameters ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultModuleParameters(
    moduleParameterId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefModuleParameters
        PRIMARY KEY(moduleParameterId))
--    CONSTRAINT FK_DefModuleParameters
--        FOREIGN KEY(moduleParameterId, defaultVersion)
--        REFERENCES ModuleParameters(moduleParameterId, version))

-- a row is always immutable
CREATE TABLE DisplayConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DisplayConfigurations
        PRIMARY KEY(configTypeId, version))

#ifColumnMissing DisplayConfigurations synchLevel
    ALTER TABLE DisplayConfigurations ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultDisplayConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefDisplayConfigurations
        PRIMARY KEY(configTypeId))
--    CONSTRAINT FK_DefDisplayConfigurations
--        FOREIGN KEY(configTypeId, defaultVersion)
--        REFERENCES DisplayConfigurations(configTypeId, version))

-- a row is always immutable
CREATE TABLE IdMaps(
    idMapId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_IdMaps
        PRIMARY KEY(idMapId, version))

#ifColumnMissing IdMaps synchLevel
    ALTER TABLE IdMaps ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultIdMaps(
    idMapId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefIdMaps
        PRIMARY KEY(idMapId))
--    CONSTRAINT FK_DefIdMaps
--        FOREIGN KEY(idMapId, defaultVersion)
--        REFERENCES IdMaps(idMapId, version))

-- a row is always immutable
CREATE TABLE UnitConversions(
    unitConversionId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_UnitConversions
        PRIMARY KEY(unitConversionId, version))

#ifColumnMissing UnitConversions synchLevel
    ALTER TABLE UnitConversions ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultUnitConversions(
    unitConversionId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefUnitConversions
        PRIMARY KEY(unitConversionId))
--    CONSTRAINT FK_DefUnitConversions
--        FOREIGN KEY(unitConversionId, defaultVersion)
--        REFERENCES UnitConversions(unitConversionId, version))

-- a row is always immutable
CREATE TABLE FlagConversions(
    flagConversionId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_FlagConversions
        PRIMARY KEY(flagConversionId, version))

#ifColumnMissing FlagConversions synchLevel
    ALTER TABLE FlagConversions ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultFlagConversions(
    flagConversionId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefFlagConversions
        PRIMARY KEY(flagConversionId))
--    CONSTRAINT FK_DefFlagConversions
--        FOREIGN KEY(flagConversionId, defaultVersion)
--        REFERENCES FlagConversions(flagConversionId, version))

-- a row is always immutable
CREATE TABLE CorrelationTravelTimes(
    travelTimesId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_CorrelationTravelTimes
        PRIMARY KEY(travelTimesId, version))

#ifColumnMissing CorrelationTravelTimes synchLevel
    ALTER TABLE CorrelationTravelTimes ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultCorrelationTravelTimes(
    travelTimesId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefCorrelationTravelTimes
        PRIMARY KEY(travelTimesId))
--    CONSTRAINT FK_DefCorrelationTravelTimes
--        FOREIGN KEY(travelTimesId, defaultVersion)
--        REFERENCES CorrelationTravelTimes(travelTimesId, version))

-- a row is always immutable
CREATE TABLE CorrelationEventSets(
    eventSetsId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_CorrelationEventSets
        PRIMARY KEY(eventSetsId, version))

#ifColumnMissing CorrelationEventSets synchLevel
    ALTER TABLE CorrelationEventSets ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultCorrelationEventSets(
    eventSetsId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefCorrelationEventSets
        PRIMARY KEY(eventSetsId))
--    CONSTRAINT FK_DefCorrelationEventSets
--        FOREIGN KEY(eventSetsId, defaultVersion)
--        REFERENCES CorrelationEventSets(eventSetsId, version))


-- a row is always immutable
CREATE TABLE CoefficientSets(
    coefficientSetsId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_CoefficientSets
        PRIMARY KEY(coefficientSetsId, version))

CREATE TABLE DefaultCoefficientSets(
    coefficientSetsId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefCoefficientSets
        PRIMARY KEY(coefficientSetsId))
--    CONSTRAINT FK_DefCoefficientSets
--        FOREIGN KEY(coefficientSetsId, defaultVersion)
--        REFERENCES CoefficientSets(coefficientSetsId, version))

-- a row is always immutable
CREATE TABLE WorkflowFiles(
    workflowId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_WorkflowFiles
        PRIMARY KEY(workflowId, version))

#ifColumnMissing WorkflowFiles synchLevel
    ALTER TABLE WorkflowFiles ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultWorkflowFiles(
    workflowId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefWorkflowFiles
        PRIMARY KEY(workflowId))
--  CONSTRAINT FK_DefWorkflowFiles
--      FOREIGN KEY(workflowId, defaultVersion)
--      REFERENCES WorkflowFiles(workflowId, version))

-- only used by MC
CREATE TABLE WorkflowTargetFsss(
    workflowId VARCHAR(64) NOT NULL,
    fssId VARCHAR(30) NOT NULL,
    CONSTRAINT PK_WorkflowTargetFsss
        PRIMARY KEY(workflowId, fssId),
    CONSTRAINT FK_WorkflowTargetFsss
        FOREIGN KEY(workflowId) REFERENCES DefaultWorkflowFiles(workflowId))

CREATE TABLE SystemActivities(
    taskRunId VARCHAR(64) NOT NULL, -- PK, this is a system activity id NOT A TASKRUN ID
    taskRunType VARCHAR(64) NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SystemActivities
        PRIMARY KEY(taskRunId))

-- this table is not used by the local datastore only by the MC and the logging module
#ifTableMissing LogEntries
CREATE TABLE LogEntries(
    logEntryId INTEGER NOT NULL, -- PK
    taskRunId VARCHAR(64) NOT NULL, -- PK, this is a system activity id NOT A TASKRUN ID
    moduleInstanceId VARCHAR(64),
    logType INTEGER NOT NULL,
    logCreationTime TIMESTAMP NOT NULL,
    logLevel INTEGER NOT NULL,
    eventCode varchar(252),
    eventAcknowledged INTEGER DEFAULT 0,
    eventProcessed INTEGER DEFAULT 0, -- todo add NOT NULL in 2012.01, is now absent to allow login to MC 2011.01
    logMessage LONGVARCHAR NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_LogEntries
        PRIMARY KEY(taskRunId, logEntryId))
-- FK to be reactivated.
--    CONSTRAINT FK_LogEntries
--        FOREIGN KEY(taskRunId)
--        REFERENCES SystemActivities(taskRunId))

-- todo document columns

CREATE INDEX LogEntries_lat ON LogEntries (localAvailableTime)

CREATE INDEX LogEntries_logCreationTime ON LogEntries (logCreationTime)

CREATE INDEX LogEntries_eventCode ON LogEntries (eventCode)

CREATE INDEX LogEntries_logLevel ON LogEntries (logLevel)

#endif

#ifColumnMissing LogEntries eventProcessed
    ALTER TABLE LogEntries ADD eventProcessed INTEGER
#endif

CREATE INDEX LogEntries_expiryTime ON LogEntries (expiryTime)

-- what if scenarios is not configuration
#ifTableMissing WhatIfScenarios
CREATE TABLE WhatIfScenarios(
    whatIfId VARCHAR(64) NOT NULL,
    description VARCHAR(255),
    creationTaskRunId VARCHAR(64),  -- this is a system activity id NOT A TASKRUN ID
    creationTime TIMESTAMP NOT NULL,
    whatIfConfig LONGVARCHAR,  -- todo add not null
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    visible TINYINT NOT NULL,
    persistent TINYINT NOT NULL,
    expiryTime TIMESTAMP,
    modificationTime TIMESTAMP NOT NULL,
    name VARCHAR(64),
    pendingDeletion TINYINT DEFAULT 0 NOT NULL,
    CONSTRAINT PK_WhatIfScenarios
        PRIMARY KEY(whatIfId))

CREATE INDEX WhatIfScenarios_lat ON WhatIfScenarios (localAvailableTime)
#endif
#ifColumnMissing  WhatIfScenarios visible
ALTER TABLE WhatIfScenarios ADD visible TINYINT NOT NULL

ALTER TABLE WhatIfScenarios ADD persistent TINYINT NOT NULL

ALTER TABLE WhatIfScenarios ADD expiryTime TIMESTAMP

ALTER TABLE WhatIfScenarios ADD  modificationTime TIMESTAMP NOT NULL
#endif

#ifColumnMissing WhatIfScenarios name
     ALTER TABLE WhatIfScenarios ADD name VARCHAR(64)
#endif

#ifColumnMissing WhatIfScenarios pendingDeletion
     ALTER TABLE WhatIfScenarios ADD pendingDeletion TINYINT DEFAULT 0 NOT NULL
#endif

CREATE INDEX WhatIfScenarios_expiryTime ON WhatIfScenarios (expiryTime)

-- a row is immutable for stand alone system because task scheduling is not supported
#ifTableMissing Tasks
CREATE TABLE Tasks(
    creationTaskRunId VARCHAR(64) NOT NULL,  --  this is a system activity id NOT A TASKRUN ID, accidently this is not part of the PK
    taskId VARCHAR(64) NOT NULL, -- PK the creationTaskRunId is prefixed to the id, this is a workaround because the creationTaskRunId is not part of the primary key
    whatIfId VARCHAR(64),
    workflowId VARCHAR(64) NOT NULL,
    description VARCHAR(255),
    ownerMcId VARCHAR(64) NOT NULL,
    canRunOnFailover TINYINT NOT NULL,
    taskPriority INTEGER NOT NULL,
    taskFirstDueTime TIMESTAMP,
    taskRepeatTime DOUBLE, -- ISSUE why double this is very dangerous why not number of millis, this is the java convention?
    taskRepeatUntilTime TIMESTAMP,
    taskStatus VARCHAR(1) NOT NULL,
    taskPendingSinceTime TIMESTAMP,
    taskTag VARCHAR(64),
    creationTime TIMESTAMP NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    taskProperties LONGVARCHAR, -- immutable
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_TASKS
        PRIMARY KEY(taskId),
    CONSTRAINT FK_TasksWhatIf
        FOREIGN KEY(whatIfId)
        REFERENCES WhatIfScenarios(whatIfId))


-- To be added later
--    CONSTRAINT FK_TasksWorkflow
--        FOREIGN KEY(workflowId)
--        REFERENCES DefaultWorkflowFiles(workflowId))

CREATE INDEX Tasks_lat ON Tasks (localAvailableTime)
#endif

CREATE INDEX Tasks_expiryTime ON Tasks (expiryTime)

#ifDatabaseType firebird
    ALTER TABLE Tasks ALTER COLUMN taskRepeatTime TYPE DOUBLE
#endif

 -- only use by MC
CREATE TABLE McCpts(
    taskRunId VARCHAR(64) NOT NULL,
    mcId VARCHAR(64) NOT NULL,
    fssId VARCHAR(30),
    cptId VARCHAR(30) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_McCpts
        PRIMARY KEY(taskRunId),
    CONSTRAINT FK_McCpts
        FOREIGN KEY(taskRunId)
        REFERENCES SystemActivities(taskRunId))

 -- only use by MC
CREATE TABLE FewsSessions(
    taskRunId VARCHAR(64) NOT NULL,  -- this is a system activity id NOT A TASKRUN ID
    ocAddress VARCHAR(64) NOT NULL,
    userId VARCHAR(30) NOT NULL,
    sessionMcId VARCHAR(64) NOT NULL,
    loginTime TIMESTAMP NOT NULL,
    lastRequestTime TIMESTAMP,
    modificationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    sessionType TINYINT NOT NULL,
    pid INTEGER,
    description VARCHAR(255),
    CONSTRAINT PK_FewsSessions
        PRIMARY KEY(taskRunId),
    CONSTRAINT FK_FewsSessions
        FOREIGN KEY(taskRunId)
        REFERENCES SystemActivities(taskRunId))

CREATE INDEX FewsSessions_expiryTime ON FewsSessions (expiryTime)

#ifColumnMissing  FewsSessions sessionType
ALTER TABLE FewsSessions ADD sessionType TINYINT NOT NULL

ALTER TABLE FewsSessions ADD pid INTEGER

ALTER TABLE FewsSessions ADD description VARCHAR(255)
#endif
-- this table is always immutable for the local data store for a online system
-- a row is always immutable for the local data store and mutable for the MC, the MC is updating the status
#ifTableMissing TaskRuns
CREATE TABLE TaskRuns(
    taskRunId VARCHAR(64) NOT NULL, -- PK this is a taskrun id and a system activity
    taskId VARCHAR(64) NOT NULL,
    taskDispatchMcId VARCHAR(64) NOT NULL, -- TODO ISSUE should be nullable null indicated MC should ignore this taskrun todo document
    taskDispatchFssId VARCHAR(30), -- * at beginning indicates MC taskruns igonored by datastore
    taskDispatchTime TIMESTAMP NOT NULL,  -- real time the taskrun is dispatched/started, always later than the scheduledDispatchTime
    taskRunStatus VARCHAR(1) NOT NULL, -- F= Failed, R = Running, C = Complete,  T = Terminated, 
    scheduledDispatchTime TIMESTAMP NOT NULL,  -- intention time to start task, used for calculation time0 for repeated task
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    modificationTime TIMESTAMP NOT NULL, -- for a stand alone system this equals the creation time because it is immutable
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_TaskRuns
        PRIMARY KEY(taskRunId),
    CONSTRAINT FK_TaskRuns
        FOREIGN KEY(taskRunId)
        REFERENCES SystemActivities(taskRunId),
    CONSTRAINT FK_TaskRunsTask
        FOREIGN KEY(taskId) REFERENCES Tasks(taskId))

CREATE INDEX TaskRuns_lat ON TaskRuns (localAvailableTime)
#endif

CREATE INDEX TaskRuns_expiryTime ON TaskRuns (expiryTime)

-- only used by local data store
-- TODO add creation time

-- todo rename modification time to creationTime  row is always readonly
-- todo add expiry time
-- todo add lastApprovedByUser

#ifTableMissing TaskRunCompletions
CREATE TABLE TaskRunCompletions(
    taskRunId VARCHAR(64) NOT NULL, -- PK this is a taskrun id and a system activity id
    taskRunCompletionStatus VARCHAR(1) NOT NULL, -- C, F,T, A (Complete, Failed, Terminated, Approved)
    taskRunCompletionTime TIMESTAMP NOT NULL,
    taskRunProperties LONGVARCHAR NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_TaskRunCompletions
        PRIMARY KEY(taskRunId),
    CONSTRAINT FK_TaskRunCompletions
        FOREIGN KEY(taskRunId)
        REFERENCES TaskRuns(taskRunId))

CREATE INDEX TaskRunCompletions_lat ON TaskRunCompletions (localAvailableTime)
#endif

CREATE TABLE ArchiveMetaData(
    taskRunId VARCHAR(64) NOT NULL, -- PK
    blob LONGVARBINARY NOT NULL,  -- compressed xml file with used rows in the database during the run
    creationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ArchiveMetaData
            PRIMARY KEY(taskRunId),
   CONSTRAINT FK_ArchiveMetaData
           FOREIGN KEY(taskRunId)
           REFERENCES TaskRunCompletions(taskRunId))



-- only used by the local data store
#ifTableMissing ModuleInstanceRuns
CREATE TABLE ModuleInstanceRuns(
    taskRunId VARCHAR(64) NOT NULL, -- task run id and a system activity id
    moduleInstanceId VARCHAR(64) NOT NULL,
    previousTaskRunId VARCHAR(64), -- task run id and a system activity id
    creationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ModuleInstanceRuns
        PRIMARY KEY(taskRunId, moduleInstanceId),
    CONSTRAINT FK_ModuleInstanceRuns
        FOREIGN KEY(taskRunId)
        REFERENCES TaskRuns(taskRunId))

CREATE INDEX ModuleInstanceRuns_lat ON ModuleInstanceRuns (localAvailableTime)
#endif

#iftablemissinganddatabasetype TimeSeries Firebird
    CREATE GENERATOR GEN_localIntId;
#endif

#iftableexists ModifiersTrial
    DROP TABLE ModifiersTrial 
#endif

-- Need to drop modifiers table, in order to add not NULL columns without default:
-- modType and priority
#ifColumnMissingTableExists Modifiers modType
    DROP TABLE Modifiers
#endif

#ifTableMissing Modifiers
CREATE TABLE Modifiers(
    taskRunId VARCHAR(64) NOT NULL, -- PK, this is a system activity id NOT A TASKRUN ID
    modifierId VARCHAR(64) NOT NULL, -- PK   // todo change to integer
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    modType VARCHAR(64) NOT NULL,
    priority INTEGER NOT NULL,
    userId VARCHAR(30) NOT NULL,
    enabled TINYINT NOT NULL,
    enabledInEnsembleRun TINYINT DEFAULT 0 NOT NULL, -- NO LONGER USED, SEE FEWS-3566
    enabledPeriodStartTime TIMESTAMP,
    enabledPeriodEndTime TIMESTAMP,
    validTime TIMESTAMP,
    visible TINYINT NOT NULL, -- false when the modifier is pending for deletion.
    creationTime TIMESTAMP NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    synchLevel INTEGER NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_Modifiers
        PRIMARY KEY(taskRunId, modifierId),
    CONSTRAINT FK_Modifiers
            FOREIGN KEY (taskRunId)
            REFERENCES SystemActivities(taskRunId))

CREATE INDEX Modifiers_synchLevel_Index ON Modifiers (synchLevel)

CREATE INDEX Modifiers_lat_Index ON Modifiers (localAvailableTime)
#endif

#ifColumnMissing Modifiers modificationTime
    ALTER TABLE Modifiers ADD modificationTime TIMESTAMP
#endif

#ifColumnMissing Modifiers enabledInEnsembleRun
    ALTER TABLE Modifiers ADD enabledInEnsembleRun TINYINT DEFAULT 0 NOT NULL
#endif

#ifColumnMissing Modifiers validTime
    ALTER TABLE Modifiers ADD validTime TIMESTAMP
#endif

CREATE INDEX Modifiers_expiryTime ON Modifiers (expiryTime)

#ifColumnExists ModuleParameterModifiers modificationTime
   ALTER TABLE ModuleParameterModifiers DROP modificationTime
#endif

#ifTableMissing ModuleParameterModifiers
CREATE TABLE ModuleParameterModifiers(
    taskRunId VARCHAR(64) NOT NULL, -- PK, this is a system activity id NOT A TASKRUN ID
    modifierId VARCHAR(64) NOT NULL, -- PK
    moduleInstanceId VARCHAR(64) NOT NULL,
    configXml LONGVARCHAR NOT NULL, 
    synchLevel INTEGER NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP NOT NULL, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ModuleParameterModifiers
        PRIMARY KEY(taskRunId, modifierId))
--  CONSTRAINT FK_ModuleParameterModifiers
--        FOREIGN KEY (taskRunId, modifierId)
--        REFERENCES Modifiers(taskRunId, modifierId)
--  ToDo activate foreign key if no problems foreseen

CREATE INDEX ParModifiers_synchLevel_Index ON ModuleParameterModifiers (synchLevel)

CREATE INDEX ParModifiers_lat_Index ON ModuleParameterModifiers (localAvailableTime)
#endif

CREATE INDEX ParModifiers_expiryTime ON ModuleParameterModifiers (expiryTime)

#ifColumnExists AttributeModifiers attributeValue
   DROP TABLE AttributeModifiers;
#endif

#ifTableMissing AttributeModifiers
CREATE TABLE AttributeModifiers(
    attributeModifierId VARCHAR(64) NOT NULL, -- PK
    taskRunId VARCHAR(64) NOT NULL, -- PK
    modifierId VARCHAR(64) NOT NULL,
    locationId VARCHAR(64) NOT NULL,
    attributeId VARCHAR(64) NOT NULL,
    attributeText VARCHAR(64),
    attributeNumber DOUBLE,
    attributeBoolean INTEGER,
    synchLevel INTEGER NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP NOT NULL, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_AttributeModifiers
        PRIMARY KEY(taskRunId, attributeModifierId),
    CONSTRAINT FK_AttribModifiersModifiers
        FOREIGN KEY (taskRunId, modifierId)
        REFERENCES Modifiers(taskRunId, modifierId))

CREATE INDEX ParAttributeModifiers_synchLevel_Index ON AttributeModifiers (synchLevel)

CREATE INDEX ParAttributeModifiers_lat_Index ON AttributeModifiers (localAvailableTime)

CREATE INDEX ParAttributeModifiers_expiryTime ON AttributeModifiers (expiryTime)

#endif

#ifDatabaseType firebird
    ALTER TABLE AttributeModifiers ALTER COLUMN attributeNumber TYPE DOUBLE
#endif


#ifTableMissing TimeSeries
CREATE TABLE TimeSeries(
    creatorTaskRunId VARCHAR(64) NOT NULL, -- PK, this is a system activity id NOT A TASKRUN ID
    blobId VARCHAR(64) NOT NULL, -- PK    TODO change to integer
    modifierId VARCHAR(64),
    sampleId VARCHAR(64),
    moduleInstanceId   VARCHAR(64) NOT NULL,
    parameterId VARCHAR(64),
    qualifierSetId VARCHAR(64),
    locationId VARCHAR(64) NOT NULL,  -- can contain multiple compressed location ids. @literal id  $id prefix  #id postfix appended to last prefix
    beginTime TIMESTAMP NOT NULL,
    endTime TIMESTAMP NOT NULL,
    timeSeriesType INTEGER NOT NULL,  -- (external historical, external forecasting, simulated historical,  simulated forecasting)
    taskRunId VARCHAR(64), -- only used for simulated forecasts TODO remove
    ensembleMemberId VARCHAR(64) NOT NULL,  -- TODO should be nullable
    blob LONGVARBINARY,  -- binary stream from where a time series array can be reconstructed
    blobSize INTEGER NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    synchLevel INTEGER NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    valueType INTEGER NOT NULL,  -- (scalar, grid, logitudinal profile, polygon)
    timeStepId VARCHAR(64) NOT NULL,  -- automatically calculated id
    aggregationPeriod VARCHAR(64),  -- automatically calculated id
    repeatCycleId VARCHAR(64),  -- automatically calculated id
    externalForecastingStartTime TIMESTAMP,  -- only used for external forcasts
    constantFlag INTEGER, -- if not null all flags in the blob are equal to the constant flag
    maxValue FLOAT, -- idem
    CONSTRAINT FK_SYSTEMACTIVITIES
            FOREIGN KEY (creatorTaskRunId)
            REFERENCES SystemActivities(taskRunId))

--    CONSTRAINT FK_TASKRUNS_ModuleInstanceRuns
--        FOREIGN KEY (creatorTaskRunId, moduleInstanceId)
--        REFERENCES ModuleInstanceRuns(taskRunId, moduleInstanceId))
--    CONSTRAINT FK_TASKRUNS_Modifiers
--        FOREIGN KEY (creatorTaskRunId, modifierId)
--        REFERENCES Modifiers(taskRunId, modifierId))
--  This constraint is not understood by access when the taskRunId is null and the moduleinstance id is not


--    CONSTRAINT PK_TimeSeries  SLOWS DOWN ACCESS dramatically blob count > 1000000
--        PRIMARY KEY(creatorTaskRunId, blobId),
--    todo constrained should be to moduleinstanceruns table

-- because an primary key with index on two string columns is two heavy for access we do an index only on the createTaskRunId column
CREATE INDEX TimeSeries_ctri_Index ON TimeSeries (creatorTaskRunId)

CREATE INDEX TimeSeries_blobId_Index ON TimeSeries (blobId)

CREATE INDEX TimeSeries_synchLevel_Index ON TimeSeries (synchLevel)

CREATE INDEX TimeSeries_lat_Index ON TimeSeries (localAvailableTime)
#endif

#ifFlagExistsAndDatabaseTypeAndColumnMissing AUTO_INCREMENT_LOCAL_INT_ID msaccess TimeSeries localIntId
    -- from 2012.01 a autoincrement local int id should be used always
    -- when this table exists in an embedded database an auto increment column is used
    -- to allow to open a local datastore in a old version we still use the local int id manager class for a while
    CREATE TABLE LocalIntIdVersion (version INTEGER NOT NULL)

    ALTER TABLE TimeSeries ADD localIntId AUTOINCREMENT NOT NULL

    -- unique is too slow in ms access
    CREATE INDEX TimeSeries_localIntId_Index ON TimeSeries (localIntId)
#endif

#ifFlagExistsAndDatabaseTypeAndColumnMissing AUTO_INCREMENT_LOCAL_INT_ID firebird TimeSeries localIntId
    -- from 2012.01 a autoincrement local int id should be used always
    -- when this table exists in an embedded database an auto increment column is used
    -- to allow to open a local datastore in a old version we still use the local int id manager class for a while
    CREATE TABLE LocalIntIdVersion (version INTEGER NOT NULL)

    ALTER TABLE TimeSeries ADD localIntId INTEGER NOT NULL

    -- unique is too slow in ms access
    CREATE INDEX TimeSeries_localIntId_Index ON TimeSeries (localIntId)

    SET GENERATOR GEN_localIntId TO 0

    CREATE TRIGGER TR_localIntId FOR TimeSeries ACTIVE BEFORE INSERT POSITION 0  AS  BEGIN if (NEW.localIntId is NULL) then NEW.localIntId=GEN_ID(GEN_localIntId, 1); END;
#endif


#ifColumnMissing TimeSeries localIntId
    ALTER TABLE TimeSeries ADD localIntId INTEGER DEFAULT -1 NOT NULL --Default to satisfy Derby

    -- unique is too slow in ms access
    CREATE INDEX TimeSeries_localIntId_Index ON TimeSeries (localIntId)
#endif


#ifColumnMissing TimeSeries modifierId
    ALTER TABLE TimeSeries ADD modifierId VARCHAR(64)
#endif

#ifColumnMissing TimeSeries sampleId
    ALTER TABLE TimeSeries ADD sampleId VARCHAR(64)
#endif

#ifColumnMissing TimeSeries qualifierSetId
    ALTER TABLE TimeSeries ADD qualifierSetId VARCHAR(64)
#endif

#ifColumnMissing TimeSeries aggregationPeriod
    ALTER TABLE TimeSeries ADD aggregationPeriod VARCHAR(64)
#endif

#ifColumnMissing TimeSeries repeatCycleId  
    ALTER TABLE TimeSeries ADD repeatCycleId VARCHAR(64)
#endif

CREATE INDEX TimeSeries_expiryTime ON TimeSeries (expiryTime)

-- only used by the local data store
#ifColumnMissingTableExists ColdStates coldstateid
   DROP TABLE ColdStates

#endif

#ifTableMissing  ColdStates
CREATE TABLE ColdStates(
    coldStateId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ColdStates
        PRIMARY KEY(coldStateId, version))

CREATE INDEX ColdStates_lat ON ColdStates (localAvailableTime)
#endif


-- a row is immutable
#ifTableMissing DefaultColdStates
CREATE TABLE DefaultColdStates(
    coldStateId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefColdStates
        PRIMARY KEY(coldStateId))
--    CONSTRAINT FK_DefColdStates
--        FOREIGN KEY(coldStateId, defaultVersion)
--        REFERENCES ColdStates(coldStateId, version))

CREATE INDEX DefColdStates_lat ON DefaultColdStates (localAvailableTime)
#endif


-- only used by the local data store
#ifTableMissing WarmStates
CREATE TABLE WarmStates(
    taskRunId VARCHAR(64) NOT NULL,
    stateId VARCHAR(64) NOT NULL, -- TODO change to integer
    name VARCHAR(255), -- todo remove
    description VARCHAR(255), -- todo remove
    moduleInstanceId VARCHAR(64) NOT NULL,
    blob LONGVARBINARY NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    stateTime TIMESTAMP NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_WarmStates
        PRIMARY KEY(taskRunId, stateId),
    CONSTRAINT FK_WarmStates
        FOREIGN KEY(taskRunId)
        REFERENCES TaskRuns(taskRunId))

CREATE INDEX WarmStates_lat ON WarmStates (localAvailableTime)
#endif

#ifConstraintExists WarmStates FK_WarmStates_ModRun
    ALTER TABLE WarmStates DROP CONSTRAINT FK_WarmStates_ModRun
#endif

#ifColumnMissing WarmStates synchLevel
    ALTER TABLE WarmStates ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE INDEX WarmStates_expiryTime ON WarmStates (expiryTime)

CREATE TABLE Reports(
    reportId VARCHAR(64) NOT NULL,
    taskRunId VARCHAR(64) NOT NULL,
    moduleInstanceId VARCHAR(64) NOT NULL,
    subjectTaskRunId VARCHAR(64) NOT NULL, -- todo remove makes no sense a report is always from the current module instance runs not from a forecast
    creationTime TIMESTAMP NOT NULL,
    blob LONGVARBINARY NOT NULL,
    subdirname VARCHAR(64) NOT NULL, -- todo remove makes no sense
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_Reports
        PRIMARY KEY(reportId, taskRunId, moduleInstanceId),
    CONSTRAINT FK_ReportsRun
        FOREIGN KEY(taskRunId, moduleInstanceId)
        REFERENCES ModuleInstanceRuns(taskRunId, moduleInstanceId))

 -- see FEWS-4602
 #ifnotdatabasetype Firebird
CREATE INDEX Reports_expiryTime ON Reports (expiryTime)
#endif

#ifTableMissing CurrentModuleInstanceRuns
CREATE TABLE CurrentModuleInstanceRuns(
    moduleInstanceId VARCHAR(64) NOT NULL,
    taskRunId VARCHAR(64) NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefModuleInstanceRuns
        PRIMARY KEY(moduleInstanceId),
    CONSTRAINT FK_DefModuleInstanceRunsDef
        FOREIGN KEY(taskRunId, moduleInstanceId)
        REFERENCES ModuleInstanceRuns(taskRunId, moduleInstanceId))

CREATE INDEX CurrentModuleInstanceRuns_lat ON CurrentModuleInstanceRuns (localAvailableTime)
#endif


CREATE TABLE Archives(
    taskRunId VARCHAR(64) NOT NULL,
    subjectTaskRunId VARCHAR(64) NOT NULL,
    subjectName VARCHAR(64),
    blob LONGVARBINARY NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_Archives
        PRIMARY KEY(taskRunId, subjectTaskRunId),
    CONSTRAINT FK_ArchivesRun
    	FOREIGN KEY(taskRunId)
    	REFERENCES SystemActivities(taskRunId))

CREATE TABLE SystemStatusDigests(
    systemStatusMcId VARCHAR(64) NOT NULL,
    systemStatusDigestId INTEGER NOT NULL,
    systemStatusDigestTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SystemStatusDigests
        PRIMARY KEY(systemStatusMcId, systemStatusDigestId))


CREATE INDEX SystemStatusDigests_expiryTime ON SystemStatusDigests (expiryTime)

CREATE TABLE SystemStatusMetrics(
    systemStatusMcId VARCHAR(64) NOT NULL,
    systemStatusDigestId INTEGER NOT NULL,
    metricId VARCHAR(64) NOT NULL,
    metricValue FLOAT NOT NULL,
    CONSTRAINT PK_systemStatusMetric
        PRIMARY KEY(systemStatusMcId, systemStatusDigestId, metricId))


#ifColumnExists AiServletUsers passwrd
    DROP TABLE AiServletUsers
#endif

CREATE TABLE AiServletUsers(
    userName VARCHAR(16) NOT NULL,
    password LONGVARBINARY NOT NULL,
    fullName VARCHAR(32) NOT NULL,
    CONSTRAINT PK_AiServletUsers
        PRIMARY KEY(userName))
        --passWord is a reserved key word for msaccess


#ifTableMissing LiveFssAvailabilities
CREATE TABLE LiveFssAvailabilities(
    mcId VARCHAR(64) DEFAULT 'none' NOT NULL,
    fssId VARCHAR(30) NOT NULL,
    queueLength INTEGER,
    CONSTRAINT PK_LiveFssAvailabilities
        PRIMARY KEY(mcId,fssId))
#endif

#ifColumnMissing LiveFssAvailabilities mcId
    ALTER TABLE LiveFssAvailabilities DROP CONSTRAINT PK_LiveFssAvailabilities

    ALTER TABLE LiveFssAvailabilities ADD mcId VARCHAR(64) DEFAULT 'none' NOT NULL

    UPDATE LiveFssAvailabilities SET mcId='none'

    ALTER TABLE LiveFssAvailabilities ADD CONSTRAINT PK_LiveFssAvailabilities PRIMARY KEY(mcId,fssId)
#endif


CREATE TABLE LiveMcAvailabilities(
    mcId VARCHAR(64) NOT NULL,
    isFailover TINYINT,
    CONSTRAINT PK_liveMcAvailabilities
        PRIMARY KEY(mcId))


CREATE TABLE LiveSystemMetrics (
    metricId VARCHAR(64) NOT NULL,
    metricValue FLOAT NOT NULL,
  CONSTRAINT PK_LiveSystemMetrics
       PRIMARY KEY(metricId))
#ifColumnMissing LiveSystemMetrics metricTime
    ALTER TABLE LiveSystemMetrics ADD metricTime TIMESTAMP 
#endif

CREATE TABLE DefaultActionConfigurations(
    actionId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefaultActionConfigurations
        PRIMARY KEY(actionId ))


CREATE TABLE ActionConfigurations(
    actionId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ActionConfigurations
        PRIMARY KEY(actionId, version),
    CONSTRAINT FK_ActionConfigurations
        FOREIGN KEY(actionId)
        REFERENCES DefaultActionConfigurations(actionId))

--ALTER TABLE DefaultActionConfigurations
--    ADD CONSTRAINT FK_DefActionConfigurationsD
--    FOREIGN KEY(actionId, defaultVersion)
--	    REFERENCES ActionConfigurations(actionId, version)

#ifColumnMissing ActionConfigurations synchLevel
    ALTER TABLE ActionConfigurations ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif


CREATE TABLE EventActionMappings(
    actionId VARCHAR(64) NOT NULL,
    eventCode VARCHAR(64) NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_EventActionMappings
        PRIMARY KEY(eventCode),
    CONSTRAINT FK_AcIdEventActionMappings
        FOREIGN KEY(actionId)
        REFERENCES DefaultActionConfigurations(actionId))

 -- only use by MC
CREATE TABLE DefaultSysReportStyles(
    id VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefSysReportStyles
        PRIMARY KEY(id))

 -- only use by MC
CREATE TABLE SysReportStyles(
    id VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SysReportStyles
        PRIMARY KEY(id, version),
    CONSTRAINT FK_SysReportStyles
        FOREIGN KEY(id)
    REFERENCES DefaultSysReportStyles(id))

--ALTER TABLE DefaultSysReportStyles
--    ADD CONSTRAINT FK_DefSysReportStylesD
--    FOREIGN KEY(Id, defaultVersion)
--    REFERENCES SysReportStyles(Id, version)

#ifColumnMissing SysReportStyles synchLevel
    ALTER TABLE SysReportStyles ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif


CREATE TABLE DefaultSysReportTemplates(
    id VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefaultSysReportTemplates
        PRIMARY KEY(id))


CREATE TABLE SysReportTemplates(
    id VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SysReportTemplates
        PRIMARY KEY(id, version),
    CONSTRAINT FK_SysReportTemplates
        FOREIGN KEY(id)
    REFERENCES DefaultSysReportTemplates(id))


--ALTER TABLE DefaultSysReportTemplates
--    ADD CONSTRAINT FK_DefSysReportTemplatesD
--    FOREIGN KEY(Id, defaultVersion)
--        REFERENCES SysReportTemplates(Id, version)


#ifColumnMissing SysReportTemplates synchLevel
    ALTER TABLE SysReportTemplates ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE SysReports(
    taskRunId VARCHAR(64) NOT NULL,
    reportMcId VARCHAR(64) NOT NULL,
    reportName VARCHAR(30) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    blob LONGVARBINARY NOT NULL,
    templateId VARCHAR(64) NOT NULL,
    templateVersion VARCHAR(64) NOT NULL,
    styleId VARCHAR(64) NOT NULL,
    styleVersion VARCHAR(64) NOT NULL,
    renderFormat TINYINT NOT NULL,
    expiryTime TIMESTAMP, -- all expired rows are invisible and removed from the datastore by the rolling barrel
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_SysReports
        PRIMARY KEY(taskRunId),
    CONSTRAINT FK_Templates
        FOREIGN KEY(templateId, templateVersion)
        REFERENCES SysReportTemplates(id, version),
    CONSTRAINT FK_Styles
        FOREIGN KEY(styleId, styleVersion)
        REFERENCES sysReportStyles(id, version),
    CONSTRAINT FK_TaskRun
        FOREIGN KEY(taskRunId)
        REFERENCES SystemActivities(taskRunId))

CREATE INDEX SysReports_expiryTime ON SysReports (expiryTime)

-- a row is always immutable
CREATE TABLE ReportTemplates(
    configTypeId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ReportTemplates
        PRIMARY KEY(configTypeId, version))

#ifColumnMissing ReportTemplates synchLevel
    ALTER TABLE ReportTemplates ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultReportTemplates(
    configTypeId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefReportTemplates
        PRIMARY KEY(configTypeId))
--    CONSTRAINT FK_DefReportTemplates
--        FOREIGN KEY(configTypeId, defaultVersion)
--        REFERENCES ReportTemplates(configTypeId, version))


#ifColumnExists ReportImages blobId
    DROP TABLE ReportImages

    DROP TABLE DefaultReportImages
#endif

-- a row is always immutable
CREATE TABLE ReportImages(
    reportImageId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL, -- revision comment (so this is not the name) that the user enters when he uploaded the file
    description VARCHAR(255) NOT NULL, -- original file name (so this is not the description) including optional sub dir(s)
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    blob LONGVARBINARY NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ReportImages
        PRIMARY KEY(reportImageId, version))

#ifColumnMissing ReportImages synchLevel
    ALTER TABLE ReportImages ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultReportImages(
    reportImageId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefReportImages
        PRIMARY KEY(reportImageId))
--    CONSTRAINT FK_DefReportImages
--        FOREIGN KEY(reportImageId, defaultVersion)
--        REFERENCES ReportImages(reportImageId, version))

-- a row is always immutable
CREATE TABLE ConfigManConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    version VARCHAR(64) NOT NULL,
    name VARCHAR(64) NOT NULL,
    description VARCHAR(255) NOT NULL,
    creationTime TIMESTAMP NOT NULL,
    creationUserId VARCHAR(30) NOT NULL,
    synchLevel INTEGER DEFAULT 0 NOT NULL,
    configXml LONGVARCHAR NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_ConfigManConfigurations
        PRIMARY KEY(configTypeId, version))

#ifColumnMissing ConfigManConfigurations synchLevel
    ALTER TABLE ConfigManConfigurations ADD synchLevel INTEGER DEFAULT 0 NOT NULL
#endif

CREATE TABLE DefaultConfigManConfigurations(
    configTypeId VARCHAR(64) NOT NULL,
    defaultVersion VARCHAR(64),
    newVersion INTEGER DEFAULT 0 NOT NULL,
    modificationTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL, -- time when the row is added to a database copy
    CONSTRAINT PK_DefConfigManConfigurations
        PRIMARY KEY(configTypeId))
--    CONSTRAINT FK_DefIcons
--        FOREIGN KEY(configTypeId, defaultVersion)
--        REFERENCES ConfigManConfigurations(configTypeId, version))

-- a row is always immutable

#iftableexists RootConfigurations
    DROP TABLE RootConfigurations

    DROP TABLE DefaultRootConfigurations
#endif

CREATE TABLE SynchTimes(
	profileId VARCHAR(64) NOT NULL,
    synchId VARCHAR(64) NOT NULL,
    synchTime TIMESTAMP NOT NULL,
    CONSTRAINT PK_McSynchTimes
        PRIMARY KEY(profileId, synchId))

-- Not used by the Master Controller
-- Used by the FewsExplorer to find the last used OC session.
-- TODO add foreign key relation to FewsSessions
CREATE TABLE CurrentSessions(
    taskRunId VARCHAR(64) NOT NULL) -- this is a system activity id NOT A TASKRUN ID

CREATE TABLE Forecasts(
    taskRunId VARCHAR(64) NOT NULL,
    expiryTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL,
    CONSTRAINT PK_Forecasts
    	PRIMARY KEY(taskRunId),
    CONSTRAINT FK_TaskRunCompletionsForecasts
	    FOREIGN KEY(taskRunId) REFERENCES TaskRunCompletions(taskRunId))

CREATE INDEX Forecasts_expiryTime ON Forecasts (expiryTime)

CREATE TABLE ThresholdEvents(
    eventEntryId INTEGER NOT NULL,
    taskRunId VARCHAR(64) NOT NULL,
    moduleInstanceId VARCHAR(64) NOT NULL,
    thresholdValueSetId VARCHAR(64) NOT NULL,
    parameterId VARCHAR(64) NOT NULL,
    locationId VARCHAR(64) NOT NULL,
    thresholdId VARCHAR(64) NOT NULL,
    rateOfChange VARCHAR(64),
    unitFromTimeStep VARCHAR(64),
    thresholdIntId INTEGER DEFAULT 0 NOT NULL,
    timeSeriesType VARCHAR(64) NOT NULL,
    eventTime TIMESTAMP NOT NULL,
    eventValue FLOAT DEFAULT 0 NOT NULL,
    eventActionId VARCHAR(64),
    eventIssued TINYINT DEFAULT 0,
    creationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP,
    localAvailableTime TIMESTAMP NOT NULL,
    modificationTime TIMESTAMP,
    taskRunIdTimeZero TIMESTAMP,
    CONSTRAINT PK_EventEntries
        PRIMARY KEY(taskRunId, eventEntryId))

#ifColumnMissing ThresholdEvents taskRunIdTimeZero
    ALTER TABLE ThresholdEvents ADD taskRunIdTimeZero TIMESTAMP
#endif

CREATE INDEX ThresholdEvents_expiryTime ON ThresholdEvents (expiryTime)

#ifColumnExists ImportStatus filesImportedCount
    DROP TABLE ImportStatus
#endif

#ifTableMissing  ImportStatus
CREATE TABLE ImportStatus(
    mcId VARCHAR(64) DEFAULT 'none' NOT NULL,
    dataFeedId VARCHAR(64) NOT NULL,
    directory VARCHAR(128) NOT NULL,
    lastImportTime TIMESTAMP NOT NULL,
    lastFileImported VARCHAR(128) NOT NULL,
    filesReadCount INTEGER NOT NULL,
    filesFailedCount INTEGER NOT NULL,
    log LONGVARCHAR,
    creationTime TIMESTAMP NOT NULL,
    localAvailableTime TIMESTAMP NOT NULL,
    modificationTime TIMESTAMP NOT NULL,
    expiryTime TIMESTAMP,
    CONSTRAINT PK_ImportStatus
        PRIMARY KEY(mcId,dataFeedId))
#endif

#ifColumnMissing ImportStatus mcId
    ALTER TABLE ImportStatus DROP CONSTRAINT PK_ImportStatus

    ALTER TABLE ImportStatus ADD mcId VARCHAR(64) DEFAULT 'none' NOT NULL

    UPDATE ImportStatus SET mcId='none'

    ALTER TABLE ImportStatus ADD CONSTRAINT PK_ImportStatus PRIMARY KEY(mcId,dataFeedId)


#endif

#ifColumnMissing ImportStatus expiryTime
    ALTER TABLE ImportStatus ADD expiryTime TIMESTAMP 
#endif

CREATE INDEX ImportStatus_expiryTime ON ImportStatus (expiryTime)

#ifTableMissing RecordsPendingDeletion
-- RecordsPendingDeletion Table 
--
-- This table stores information on what records have been marked 
-- by the system as requiring deletion.
CREATE TABLE RecordsPendingDeletion (
  pendingDeletionId VARCHAR(64) NOT NULL,
  tableName VARCHAR(64) NOT NULL, 
  recordKey LONGVARBINARY NOT NULL,
  creationTime TIMESTAMP NOT NULL,
  expiryTime TIMESTAMP NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  CONSTRAINT PK_RecordsPendingDeletion
      PRIMARY KEY(pendingDeletionId))
#endif

CREATE INDEX RecordsPendingDel_expiryTime ON RecordsPendingDeletion (expiryTime)

#ifTableMissing PurgeTimes
-- PurgeTimes Table 
--
-- This table stores information on when the last time the
-- records pending deletion were purged from the local database.
CREATE TABLE PurgeTimes (
 PurgeTimeId   VARCHAR(64) NOT NULL,
 PurgeTime    TIMESTAMP NOT NULL,
 CONSTRAINT PK_PurgeTimes
      PRIMARY KEY(PurgeTimeId))
 #endif
 
#ifColumnExists ConfigRevisions local_available_time
    DROP TABLE ConfigRevisionStore
    
    DROP TABLE ConfigRevisions
#endif
 
#ifTableMissing ConfigRevisions
-- ConfigRevisions Table
--
-- Not used by the Master Controller
CREATE TABLE ConfigRevisions (
  revisionId          VARCHAR(64) NOT NULL,
  creationTime        TIMESTAMP NOT NULL,
  creationUserId     VARCHAR(30) NOT NULL,
  commentText         VARCHAR(254),
  numberOfChanges    INTEGER NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  CONSTRAINT PK_ConfigRevisions
      PRIMARY KEY(revisionId));
      
#endif

#ifTableMissing ConfigRevisionStore
-- ConfigRevisionStore Table
--
-- Not used by the Master Controller
-- version set nullable due to Oracle behaviour with empty strings
CREATE TABLE ConfigRevisionStore (
  configTableName    VARCHAR(64) NOT NULL,
  configTypeId       VARCHAR(64) NOT NULL,
  version              VARCHAR(64),
  revisionId          VARCHAR(64) NOT NULL,
  creationTime        TIMESTAMP NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  CONSTRAINT PK_ConfigRevisionStore
        PRIMARY KEY(configTableName, configTypeId, revisionId),
  CONSTRAINT FK_ConfigRevisions
        FOREIGN KEY(revisionId)
        REFERENCES ConfigRevisions(revisionId));

#endif

-- Synchronisation 2.1
#ifTableMissing GlobalSynchronisationVersion
-- GlobalSynchronisationVersion table
--
CREATE TABLE GlobalSynchronisationVersion (
  id                 INTEGER NOT NULL,
  synchVersion       BIGINT NOT NULL,
  CONSTRAINT PK_GlobalSynchVersion
       PRIMARY KEY(id));
#endif

#ifTableMissing TableSynchronisationVersion
-- TableSynchronisationVersion table
--
CREATE TABLE TableSynchronisationVersion (
  id                 INTEGER NOT NULL,
  tableName          VARCHAR(255) NOT NULL,
  direction          VARCHAR(255) NOT NULL,
  synchVersion       BIGINT NOT NULL,
  synchVersionDate   TIMESTAMP NOT NULL,
  CONSTRAINT PK_TableSynchVersion
       PRIMARY KEY(id));
#endif

#ifTableMissing TableDeletion
-- TableDeletion table
--
CREATE TABLE TableDeletion (
  id                 INTEGER NOT NULL,
  tableClassName     VARCHAR(255) NOT NULL,
  synchVersion       BIGINT NOT NULL,
  primaryKey         LONGVARBINARY NOT NULL,
  primaryKeyString   VARCHAR(255) NOT NULL,
  CONSTRAINT PK_TableDeletion
       PRIMARY KEY(id));
#endif

#ifTableMissing RecordsPendingUpdateExpiry
-- RecordsPendingUpdateExpiry Table 
--
-- This table stores information on what forecasts have been marked 
-- by the system as requiring an update of the expiryTime.
CREATE TABLE RecordsPendingUpdateExpiry (
  pendingUpdateExpiryId VARCHAR(64) NOT NULL,
  taskRunId VARCHAR(64),
  taskId VARCHAR(64),
  whatIfId VARCHAR(64),
  modifierId VARCHAR(64),
  newExpiryTime TIMESTAMP NOT NULL,
  timeSeriesType INTEGER,
  creationTime TIMESTAMP NOT NULL,
  expiryTime TIMESTAMP NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  CONSTRAINT PK_RecordsPendingUpdateExpiry
      PRIMARY KEY(pendingUpdateExpiryId));
#endif

#ifColumnMissing RecordsPendingUpdateExpiry modifierId
    ALTER TABLE RecordsPendingUpdateExpiry ADD modifierId VARCHAR(64)
#endif

#ifColumnMissing RecordsPendingUpdateExpiry timeSeriesType
    ALTER TABLE RecordsPendingUpdateExpiry ADD timeSeriesType INTEGER
#endif

CREATE INDEX RecordsPenUpdateExp_expiryTime ON RecordsPendingUpdateExpiry (expiryTime)

#ifTableMissing LastDatabaseConnection
CREATE TABLE LastDatabaseConnection(
    startTime TIMESTAMP,
    endTime TIMESTAMP)

    INSERT INTO LastDatabaseConnection VALUES(NULL, NULL)
#endif

#ifTableMissing LastDatabaseCompact
    CREATE TABLE LastDatabaseCompact(
      startTime TIMESTAMP,
      endTime TIMESTAMP,
      startSize BIGINT,
      endSize BIGINT
    )

--  This statement is accidentally removed when the PiClientDataSets are added.
--  This LastDatabaseCompactTable#ensureSingleRow tries to detect and fix this damage

    INSERT INTO LastDatabaseCompact VALUES(NULL, NULL, NULL, NULL)
#endif

#ifTableMissing PiClientDataSets
CREATE TABLE PiClientDataSets(
 dataSetId VARCHAR(64) NOT NULL,
  nodeId   VARCHAR(64) NOT NULL,
  clientId VARCHAR(64) NOT NULL,
  description VARCHAR(255) NOT NULL,
  creationTime TIMESTAMP NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  modificationTime TIMESTAMP NOT NULL,
  dataSet LONGVARBINARY NOT NULL,
  visible TINYINT NOT NULL,
   CONSTRAINT PK_PiClientDataSets
      PRIMARY KEY(dataSetId,clientid));
#endif


#ifTableMissing ConfigRevisionSets
-- ConfigRevisionSets Table is the 2011_01 replacement for ConfigRevisions and ConfigRevisionStore. 
-- Whereas ConfigRevisions and ConfigRevisionStore specify revisions by a new record in the ConfigRevisionStore for each change,
-- and storing the number of changes in the configrevisions table, this unfortunately produces too many records in the 
-- ConfigRevisionStore table for for instance NWS. This causes the synchronisation to become very slow.
-- ConfigRevisions and ConfigRevisionStore can not be dropped in this stage.
-- ConfigRevisionSets contains the same information, but maintains a list of references of all config files and their versions,
-- Data from 2011_01 ConfigRevisions and ConfigRevisionStore data is to be migrated to ConfigRevisionSets using the ConfigManager.
-- When all clients have migrated, database tables ConfigRevisions and ConfigRevisionStore can eventually be dropped. 
CREATE TABLE ConfigRevisionSets (
  revisionId          VARCHAR(64) NOT NULL,
  creationTime        TIMESTAMP NOT NULL,
  creationUserId     VARCHAR(30) NOT NULL,
  commentText         VARCHAR(254),
  numberOfChanges    INTEGER NOT NULL,
  localAvailableTime TIMESTAMP NOT NULL,
  dataSet LONGVARBINARY,
  CONSTRAINT PK_ConfigRevisionSets
      PRIMARY KEY(revisionId));
      
#endif
